@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of HAPI.
 //
@@ -53,9 +53,9 @@ OpenHapticsRenderer::renderer_registration(
                             &(newInstance< OpenHapticsRenderer >)
                             );
 namespace OpenHapticsRendererInternals {
-  string getHLErrorString( HLerror error ) {
+  std::string getHLErrorString( HLerror error ) {
     if( error.errorCode == HL_DEVICE_ERROR ) {
-      stringstream s;
+      std::stringstream s;
       s << "HL_DEVICE_ERROR( " 
         << hdGetErrorString( error.errorInfo.errorCode )
         << " )";
@@ -68,7 +68,7 @@ namespace OpenHapticsRendererInternals {
 
 // Initialize static variables.
 int OpenHapticsRenderer::nr_of_context = 0;
-auto_ptr< H3DUtil::PeriodicThread > OpenHapticsRenderer::data_update_thread(0);
+std::auto_ptr< H3DUtil::PeriodicThread > OpenHapticsRenderer::data_update_thread(0);
 H3DUtil::MutexLock OpenHapticsRenderer::data_update_lock;
 HAPIInt32 OpenHapticsRenderer::nr_of_data_update_callbacks = 0;
 
@@ -415,6 +415,7 @@ void OpenHapticsRenderer::preProcessShapes( HAPIHapticsDevice *hd,
           hlPopAttrib();
 #endif
 #ifdef HAVE_OPENGL
+          glMatrixMode( GL_MODELVIEW );
           glPopMatrix();
 #endif
           if( disable_gl_modelview ) {
@@ -422,8 +423,8 @@ void OpenHapticsRenderer::preProcessShapes( HAPIHapticsDevice *hd,
             hlPopMatrix();
           }
       } else {
-        H3DUtil::Console(2) 
-          <<  "Surface type not supported by OpenHapticsRenderer." << endl;
+        H3DUtil::Console(H3DUtil::LogLevel::Debug)
+          <<  "Surface type not supported by OpenHapticsRenderer." << std::endl;
       }
     } 
   }
@@ -461,8 +462,8 @@ void OpenHapticsRenderer::preProcessShapes( HAPIHapticsDevice *hd,
   // check for any errors
   HLerror error;
   while ( HL_ERROR(error = hlGetError()) ) {
-    H3DUtil::Console(4) <<
-      OpenHapticsRendererInternals::getHLErrorString( error ) << endl;
+    H3DUtil::Console(H3DUtil::LogLevel::Error) <<
+      OpenHapticsRendererInternals::getHLErrorString( error ) << std::endl;
   }
 }
 
@@ -572,14 +573,15 @@ void HLCALLBACK OpenHapticsRenderer::touchCallback( HLenum event,
 
   OpenHapticsRenderer *renderer = cb_data->renderer;
   HAPIHapticShape *shape = cb_data->shape.get();
+  if( !shape ) return;
 
-  vector< int >::iterator found_id =
+  std::vector< int >::iterator found_id =
     find( renderer->already_removed_id.begin(),
           renderer->already_removed_id.end(),
           shape->getShapeId() );
   if( found_id == renderer->already_removed_id.end() ) {  
     renderer->tmp_contacts.push_back(
-      make_pair( H3DUtil::AutoRef< HAPIHapticShape >(shape), HAPISurfaceObject::ContactInfo()) );
+      std::make_pair( H3DUtil::AutoRef< HAPIHapticShape >(shape), HAPISurfaceObject::ContactInfo()) );
     OpenHapticsRenderer::motionCallback( event, object,
                                          thread, cache, userdata );
   } else {
@@ -618,7 +620,7 @@ void HLCALLBACK OpenHapticsRenderer::untouchCallback( HLenum event,
 
 HLuint OpenHapticsRenderer::getHLShapeId( HAPIHapticShape *hs,
                                           HAPIHapticsDevice *hd ) {
-  pair< int, HAPIHapticsDevice * > key = make_pair( hs->getShapeId(), hd );
+  std::pair< int, HAPIHapticsDevice * > key = std::make_pair( hs->getShapeId(), hd );
 
   if( id_map.find( key ) == id_map.end() ) {
     HLuint hl_shape_id = hlGenShapes(1);
