@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of HAPI.
 //
@@ -32,7 +32,7 @@
 #include <H3DUtil/H3DMath.h>
 
 using namespace HAPI;
-
+using namespace std;
 
 HAPIHapticsRenderer::HapticsRendererRegistration 
 GodObjectRenderer::renderer_registration(
@@ -44,11 +44,6 @@ namespace GodObjectRendererConstants {
 
   const Vec3 UNINITIALIZED_PROXY_POS = Vec3( -200, -200, -202 );
 
-  // the minimum distance the proxy will be from the surface. The proxy
-  // will be moved above the surface with the given factor in order to
-  // avoid roundoff errors and fallthrough problems.
-  const HAPIFloat min_distance = 1e-7;
-
   // epsilon value for deciding if a point is the same
   const HAPIFloat length_sqr_point_epsilon = 1e-15; //12
 
@@ -56,8 +51,9 @@ namespace GodObjectRendererConstants {
   const HAPIFloat length_sqr_normal_epsilon = 1e-15;
 }
 
-GodObjectRenderer::GodObjectRenderer():
-  proxy_position( GodObjectRendererConstants::UNINITIALIZED_PROXY_POS ) {
+GodObjectRenderer::GodObjectRenderer( HAPIFloat _min_distance, bool _secondary_collisions):
+  proxy_position( GodObjectRendererConstants::UNINITIALIZED_PROXY_POS ),
+  min_distance ( _min_distance ), secondary_collisions ( _secondary_collisions ) {
 }
 
 inline bool planeIntersect( Vec3 p1, Vec3 n1, Vec3 p2, Vec3 n2,
@@ -116,7 +112,8 @@ inline Vec3 projectOntoPlaneIntersection( const Vec3 &p,
 
 inline Vec3 moveAboveConstraints(  
           const Vec3 &pos,                                        
-          vector< PlaneConstraint > &constraints ) {
+          vector< PlaneConstraint > &constraints,
+          HAPIFloat min_distance) {
   Vec3 moved_pos = pos;
   bool done = false;
   int counter = 0;
@@ -126,7 +123,7 @@ inline Vec3 moveAboveConstraints(
          i != constraints.end(); ++i ) {
       HAPIFloat d = (*i).normal * (moved_pos - (*i).point );
       if( d <= 0 ) {
-        moved_pos = moved_pos + (*i).normal * (-d+GodObjectRendererConstants::min_distance);
+        moved_pos = moved_pos + (*i).normal * (-d+min_distance);
         done = false;
       }
     }
@@ -176,7 +173,7 @@ void GodObjectRenderer::onOnePlaneContact(
 
   Vec3 new_proxy_pos = 
     contact.contact_point_global + 
-    contact.y_axis * GodObjectRendererConstants::min_distance +
+    contact.y_axis * min_distance +
     contact.proxy_movement_local.x * contact.x_axis + 
     contact.proxy_movement_local.y * contact.z_axis;
 
@@ -210,6 +207,7 @@ void GodObjectRenderer::onTwoPlaneContact(
   
   Vec3 contact_global = contact.globalContactPoint();
 
+
   // the direction along the intersection of the two planes. 
   Vec3 line_dir = p0.normal % p1.normal;
   line_dir.normalizeSafe();
@@ -295,7 +293,7 @@ void GodObjectRenderer::onTwoPlaneContact(
  
     Vec3 new_proxy_pos = 
       contact.contact_point_global + 
-      contact.y_axis * GodObjectRendererConstants::min_distance +
+      contact.y_axis * min_distance +
       contact.proxy_movement_local.x * contact.x_axis + 
       contact.proxy_movement_local.y * contact.z_axis;
 
@@ -412,7 +410,7 @@ void GodObjectRenderer::onThreeOrMorePlaneContact(
 
     contact.x_axis.normalizeSafe();
     contact.z_axis.normalizeSafe();
-    contact.setGlobalOrigin( contact.contact_point_global + GodObjectRendererConstants::min_distance * contact.y_axis );
+    contact.setGlobalOrigin( contact.contact_point_global + min_distance * contact.y_axis );
 
     // calculate the force and proxy movement for the first plane
     assert( p0.haptic_shape.get() );
@@ -494,7 +492,7 @@ GodObjectRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
       // point to which the proxy has been moved by the shape itself.
       if( (*i)->lineIntersect( moved_proxy_pos, moved_proxy_pos, intersection,
                                (*i)->getTouchableFace(), true ) ){
-        moved_proxy_pos = intersection.point + GodObjectRendererConstants::min_distance * intersection.normal;
+        moved_proxy_pos = intersection.point + min_distance * intersection.normal;
         // add an extra constraint for this shape at the new position. It might not have
         // been intersected last loop and ignoring it might cause fallthrough because
         // it will not be present when we move proxy above all constraint planes.
@@ -515,7 +513,7 @@ GodObjectRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
     // make sure the new proxy position is above the constraint planes
     // from the last loop in order to prevent fallthrough.
     moved_proxy_pos = 
-      moveAboveConstraints( moved_proxy_pos, all_constraints );
+      moveAboveConstraints( moved_proxy_pos, all_constraints, min_distance );
   }
 
   proxy_pos = moved_proxy_pos;
@@ -560,7 +558,6 @@ GodObjectRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
         // first shape that was intersected, and therefore also the
         // closest intersection
         closest_intersection = intersection;
-        Vec3 v = intersection.point - proxy_pos;
         has_intersection = true;
         closest_constraints.push_back( pc  );
       } else {
@@ -616,7 +613,6 @@ GodObjectRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
   contact.tex_coord = closest_intersection.tex_coord;
 
   if( nr_constraints == 0 ) {
-    // TODO: 0.05??
     new_proxy_pos = proxy_pos + (input.position-proxy_pos)*0.05;
   } else {
     if( nr_constraints == 1 ) {
@@ -704,7 +700,7 @@ bool GodObjectRenderer::tryProxyMovement( Vec3 from, Vec3 to,
         // the from_point to an arbitrary position in the direction of the
         // closest_intersection normal.
         plane.normal = normal;
-        plane.point = closest_intersection.point + normal * GodObjectRendererConstants::min_distance;
+        plane.point = closest_intersection.point + normal * min_distance;
         Collision::IntersectionInfo intersection;
         if( plane.lineIntersect( from_point,
                                  closest_intersection.point,
@@ -712,49 +708,46 @@ bool GodObjectRenderer::tryProxyMovement( Vec3 from, Vec3 to,
           from_point = intersection.point;
         }
 
-        // TODO:This code will fix some fall through problems that occur
-        // sometimes but the cost is at the moment to great to introduce this.
-        // Needs to be evaluated on a faster computer.
-        //Vec3 old_from_point = from_point;
+        Vec3 old_from_point = from_point;
 
         // project onto plane intersection between intersection plane and
         // closest constraint.
         from_point =  
           projectOntoPlaneIntersection( from_point, 
                                         closest_intersection.point + 
-                                        normal * GodObjectRendererConstants::min_distance,
+                                        normal * min_distance,
                                         normal,
                                         pc.point + 
-                                        pc.normal * GodObjectRendererConstants::min_distance,
+                                        pc.normal * min_distance,
                                         pc.normal );
 
-        // This code will fix some fall through problems that occur sometimes
-        // But the cost is at the moment to great to introduce this.
-        /*for( HapticShapeVector::const_iterator i = shapes.begin();
-          i != shapes.end();
-          ++i ) {
+        if (secondary_collisions) {
+          for (HapticShapeVector::const_iterator i = shapes.begin();
+            i != shapes.end();
+            ++i) {
             Collision::IntersectionInfo intersection;
-            if( (*i)->lineIntersect( old_from_point, from_point, intersection,
-              (*i)->getTouchableFace()) ) {
-                from_point = old_from_point;
-                break;
+            if ((*i)->lineIntersect(old_from_point, from_point, intersection,
+              (*i)->getTouchableFace())) {
+              from_point = old_from_point;
+              break;
             }
-        }*/
+          }
+        }
 
         to_point = 
           projectOntoPlaneIntersection( to_point, 
                                         closest_intersection.point + 
-                                        normal * GodObjectRendererConstants::min_distance,
+                                        normal * min_distance,
                                         normal,
                                         pc.point + 
-                                        pc.normal * GodObjectRendererConstants::min_distance,
+                                        pc.normal * min_distance,
                                         pc.normal );
         have_new_goal = true;
       } else {
         Vec3 move = closest_intersection.point - from_point;
         HAPIFloat l = move.length();
-        if( l >= GodObjectRendererConstants::min_distance + 1e-11) {
-          move = move * ( (l-GodObjectRendererConstants::min_distance) / l );
+        if( l >= min_distance + 1e-11) {
+          move = move * ( (l-min_distance) / l );
           point = from_point + move;
         } else {
           point = from_point;
