@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of HAPI.
 //
@@ -29,15 +29,12 @@
 
 #include <HAPI/HAPIHapticsDevice.h>
 #include <HAPI/GodObjectRenderer.h>
-
-#ifdef HAVE_PROFILER
 #include <H3DUtil/H3DTimer.h>
-#endif
 
 using namespace HAPI;
 
 HAPIHapticsDevice::local_auto_ptr< 
-    list< HAPIHapticsDevice::HapticsDeviceRegistration > 
+    std::list< HAPIHapticsDevice::HapticsDeviceRegistration > 
   > HAPIHapticsDevice::registered_devices( NULL );
  
 bool HAPIHapticsDevice::initialized = false; 
@@ -54,10 +51,11 @@ H3DUtil::PeriodicThread::CallbackCode
     static_cast< HAPIHapticsDevice * >( data );
 
   // update last transform for all shapes.
-  // TODO: do this much faster/better
   for( unsigned int layer = 0; layer < hd->tmp_shapes.size(); ++layer ) {
+    H3DTIMER_BEGIN("TransferObjectPerLayer");
     for( HapticShapeVector::const_iterator s = hd->tmp_shapes[layer].begin(); 
          s != hd->tmp_shapes[layer].end(); ++s ) {
+      H3DTIMER_BEGIN("TransferObjectPerShape");
       HAPIHapticShape *shape = NULL;
       if( hd->current_shapes.size() > layer ) {
         for( HapticShapeVector::const_iterator current_s =
@@ -72,7 +70,9 @@ H3DUtil::PeriodicThread::CallbackCode
       // if we have a new shape we need to initialize it.
       if( (*s) != shape ) 
         (*s)->initializeTransfer( shape );
+      H3DTIMER_END("TransferObjectPerShape");
     }
+    H3DTIMER_END("TransferObjectPerLayer");
   }
 
   // shapes
@@ -82,6 +82,7 @@ H3DUtil::PeriodicThread::CallbackCode
   // should be added with interpolation in some cases.
   bool already_transferred = false;
   // If set effects was called.
+  H3DTIMER_BEGIN("HANDLE_SWITCHING_EFFECTS");
   if( hd->tmp_switching_effects ) {
     if( hd->tmp_switch_effects_duration > Constants::epsilon ) {
       hd->last_add_rem_effect_map.clear();
@@ -100,11 +101,13 @@ H3DUtil::PeriodicThread::CallbackCode
     hd->current_force_effects = hd->tmp_current_force_effects;
     already_transferred = true;
   }
-
+  H3DTIMER_END("HANDLE_SWITCHING_EFFECTS");
+  H3DTIMER_BEGIN("HANDLE_ADDEFFECT");
   // If addEffect was called
   if( !hd->added_effects_indices.empty() ) {
-    for( vector< pair< unsigned int, HAPITime > >::iterator i =
-           hd->added_effects_indices.begin();
+    
+    for( std::vector< std::pair< unsigned int, HAPITime > >::iterator i =
+         hd->added_effects_indices.begin();
          i != hd->added_effects_indices.end(); ++i ) {
       unsigned int index = (*i).first;
       if( !already_transferred ) {
@@ -120,10 +123,11 @@ H3DUtil::PeriodicThread::CallbackCode
     }
     hd->added_effects_indices.clear();
   }
-
+  H3DTIMER_END("HANDLE_ADDEFFECT");
+  H3DTIMER_BEGIN("HANDLE_REMOVEEFFECT");
   // If removeEffect was called.
   if( !hd->force_effects_to_remove.empty() ) {
-    for( vector< pair< HAPIForceEffect *, HAPITime > >::iterator i =
+    for( std::vector< std::pair< HAPIForceEffect *, HAPITime > >::iterator i =
          hd->force_effects_to_remove.begin();
          i != hd->force_effects_to_remove.end(); ++i ) {
 
@@ -177,6 +181,7 @@ H3DUtil::PeriodicThread::CallbackCode
     }
     hd->force_effects_to_remove.clear();
   }
+  H3DTIMER_END("HANDLE_REMOVEEFFECT");
 #ifdef HAVE_PROFILER
   H3DUtil::H3DTimer::stepEnd("transferObject_begin");
   std::stringstream temp_profiledResult;
@@ -184,10 +189,18 @@ H3DUtil::PeriodicThread::CallbackCode
   H3DUtil::H3DTimer::end("haptic_profile_transferObject",temp_profiledResult);
 
   hd->profiled_result_lock.lock();
-  hd->profiled_result_haptic[0] = temp_profiledResult.str();
+  hd->profiled_result_haptic[0] = "timer_for_objectTransfer: \n"+temp_profiledResult.str();
   hd->profiled_result_lock.unlock();
   //std::cout<<"hatpic profile result:"<<temp<<std::endl;
 #endif
+  
+  // Signal that transfer obejcts callback is done
+  // Wake up threads that are waiting to write to the shared data
+  hd->transfer_objects_cond.lock ();
+  hd->transfer_objects_active= false;
+  hd->transfer_objects_cond.signal ();
+  hd->transfer_objects_cond.unlock ();
+
   return H3DUtil::PeriodicThread::CALLBACK_DONE;
 }
 #ifdef HAVE_PROFILER
@@ -229,7 +242,7 @@ H3DUtil::PeriodicThread::CallbackCode
 
   hd->updateDeviceValues( dt );
   
-  HAPIForceEffect::EffectOutput output;
+  HAPIForceEffect::EffectOutput _output;
   
   bool last_add_interpolate = !hd->last_add_rem_effect_map.empty();
   bool current_add_interpolate = !hd->current_add_rem_effect_map.empty();
@@ -241,7 +254,7 @@ H3DUtil::PeriodicThread::CallbackCode
 
     // If any force effect should be removed from last_force_effect it is
     // stored here. Only used if last_add_interpolate is true.
-    list< HAPIForceEffect * > to_remove;
+    std::list< HAPIForceEffect * > to_remove;
     TimeStamp now_time;
 
     // calculate the forces generated by the last active force effects.
@@ -269,10 +282,10 @@ H3DUtil::PeriodicThread::CallbackCode
                 hd->last_add_rem_effect_map[ j ];
               hd->last_add_rem_effect_map.erase( j );
             }
-            output = output + (*i)->calculateForces( input ) * temp_fraction;
+            _output = _output + (*i)->calculateForces( input ) * temp_fraction;
           }
         } else {
-          output = output + (*i)->calculateForces( input );
+          _output = _output + (*i)->calculateForces( input );
         }
       }
     } else {
@@ -281,7 +294,7 @@ H3DUtil::PeriodicThread::CallbackCode
            hd->last_force_effects.begin();
          i != hd->last_force_effects.end();
          ++i ) {
-        output = output + (*i)->calculateForces( input );
+        _output = _output + (*i)->calculateForces( input );
       }
     }
 
@@ -298,13 +311,13 @@ H3DUtil::PeriodicThread::CallbackCode
         hd->last_add_rem_effect_map.clear();
         hd->switching_effects = false;
       }
-      output = output * ( 1 - force_interpolation_fraction );
+      _output = _output * ( 1 - force_interpolation_fraction );
     }
 
     if( !hd->last_force_effects.empty() ) {
       // remove HAPIForceEffects from last_force_effects if they have been
       // phased out.
-      for( list< HAPIForceEffect * >::iterator i = to_remove.begin();
+      for( std::list< HAPIForceEffect * >::iterator i = to_remove.begin();
            i != to_remove.end();
            ++i ) {
         hd->last_force_effects.erase( *i );
@@ -329,11 +342,11 @@ H3DUtil::PeriodicThread::CallbackCode
             temp_fraction = 1;
             hd->current_add_rem_effect_map.erase( found );
           }
-          output = output + (*i)->calculateForces( input ) *
+          _output = _output + (*i)->calculateForces( input ) *
                             temp_fraction *
                             force_interpolation_fraction;
         } else
-          output = output + (*i)->calculateForces( input ) *
+          _output = _output + (*i)->calculateForces( input ) *
                             force_interpolation_fraction;
       }
     } else {
@@ -342,7 +355,7 @@ H3DUtil::PeriodicThread::CallbackCode
            hd->current_force_effects.begin();
          i != hd->current_force_effects.end();
          ++i ) {
-        output = output + (*i)->calculateForces( input )
+        _output = _output + (*i)->calculateForces( input )
           * force_interpolation_fraction;
       }
     }
@@ -353,7 +366,7 @@ H3DUtil::PeriodicThread::CallbackCode
            hd->current_force_effects.begin();
          i != hd->current_force_effects.end();
          ++i ) {
-      output = output + (*i)->calculateForces( input );
+      _output = _output + (*i)->calculateForces( input );
     }
   }
 
@@ -368,7 +381,7 @@ H3DUtil::PeriodicThread::CallbackCode
       if( s + 1 > hd->current_shapes.size() )
         hd->current_shapes.resize( s + 1 );
 
-      output = output + 
+      _output = _output + 
         hd->haptics_renderers[s]->
         renderHapticsOneStep( hd, hd->current_shapes[s], dt );
     }
@@ -376,27 +389,31 @@ H3DUtil::PeriodicThread::CallbackCode
 
   hd->renderer_change_lock.unlock();
 
-  // clamp to limits  
+  // scale output force
+  _output.force = _output.force * hd->ts_force_scale;
+
+  // clamp to limits
   HAPIFloat max_force = hd->getForceLimit();
   HAPIFloat max_torque = hd->getTorqueLimit();
 
   if( max_force >= 0 ) {
-    HAPIFloat length_sqr = output.force.lengthSqr();
+    HAPIFloat length_sqr = _output.force.lengthSqr();
     if( length_sqr > max_force * max_force ) {
-      output.force = output.force * (max_force / H3DUtil::H3DSqrt( length_sqr ) );
+      _output.force = _output.force * (max_force / H3DUtil::H3DSqrt( length_sqr ) );
     }
   }
 
   if( max_torque >= 0 ) {
-    HAPIFloat length_sqr = output.torque.lengthSqr();
+    HAPIFloat length_sqr = _output.torque.lengthSqr();
     if( length_sqr > max_torque * max_torque ) {
-      output.torque = output.torque * (max_torque / H3DUtil::H3DSqrt( length_sqr ) );
+      _output.torque = _output.torque * (max_torque / H3DUtil::H3DSqrt( length_sqr ) );
     }
   }
 
   // add the resulting force and torque to the rendered force.
-  hd->sendForce( output.force );
-  hd->sendTorque( output.torque );
+  hd->sendForce( _output.force );
+  hd->sendTorque( _output.torque );
+  hd->sendRawDof7Force( _output.dof7_force );
 
   hd->sendOutput( dt );
 
@@ -419,22 +436,42 @@ H3DUtil::PeriodicThread::CallbackCode
   
   H3DUtil::H3DTimer::end("haptic_profile",temp);
   hd->profiled_result_lock.lock();
-  hd->profiled_result_haptic[1] = temp.str();
+  hd->profiled_result_haptic[1] ="timer_for_haptic_rendering: \n" + temp.str();
   hd->profiled_result_lock.unlock();
 #endif
   return H3DUtil::PeriodicThread::CALLBACK_CONTINUE;
 }
 
 void HAPIHapticsDevice::transferObjects() {
+  
   if( thread ) {
+
+    // Wait for previous callback to complete before starting next
+    transfer_objects_cond.lock ();
+    
+    while ( transfer_objects_active ) {
+      transfer_objects_cond.wait();
+    }
+
+    if ( !haveObjectsChanged() ) {
+      transfer_objects_cond.unlock ();
+      return;
+    }
+    
+    transfer_objects_active= true;
+    transfer_objects_cond.unlock ();
+
     for( unsigned int s = 0; s < haptics_renderers.size(); ++s ) {
+      H3DTIMER_BEGIN("TRANSFEROBJECT_preprocessShape");
       if( haptics_renderers[s] && s < tmp_shapes.size() ) {
          haptics_renderers[s]->preProcessShapes( this, tmp_shapes[s] );
       }
+      H3DTIMER_END("TRANSFEROBJECT_preprocessShape");
     }
-    thread->synchronousCallback( transferObjectsCallback,
-                                 this );
 
+    H3DTIMER_BEGIN("transferObjectCallback");
+    thread->asynchronousCallback( transferObjectsCallback, this);
+    H3DTIMER_END("transferObjectCallback");
   }
 }
 
@@ -479,11 +516,13 @@ HAPIHapticsDevice::ErrorCode HAPIHapticsDevice::initDevice(
 }
 
 void HAPIHapticsDevice::updateDeviceValues( HAPITime dt ) {
+
   if( device_state == ENABLED ) {
     DeviceValues dv;
     updateDeviceValues( dv, dt );
     
     device_values_lock.lock();
+    ts_force_scale = force_scale;
     last_device_values = current_device_values;
     last_raw_device_values = current_raw_device_values;
     current_raw_device_values = dv;
@@ -499,13 +538,54 @@ void HAPIHapticsDevice::updateDeviceValues( HAPITime dt ) {
     current_device_values.torque = 
       position_calibration.getScaleRotationPart() * dv.torque;
     current_device_values.button_status = dv.button_status;
+    current_device_values.angular_velocity = 
+      orientation_calibration * dv.angular_velocity;
+    current_device_values.dof7_angle = dv.dof7_angle;
+    current_device_values.dof7_force = dv.dof7_force;
     device_values_lock.unlock();
   } else {
     if( device_state == INITIALIZED ) {
       device_values_lock.lock();
+      ts_force_scale = force_scale;
       last_device_values = current_device_values;
       last_raw_device_values = current_raw_device_values;
       device_values_lock.unlock();
     }
   }
 }
+
+bool HAPIHapticsDevice::haveObjectsChanged () {
+  if ( always_transfer_objects ) {
+    return true;
+  }
+
+  // Compare force effects
+  if ( tmp_current_force_effects.size() == current_force_effects.size() ) {
+    for ( size_t i= 0; i < tmp_current_force_effects.size(); ++i ) {
+      if ( tmp_current_force_effects[i] != current_force_effects[i] ) {
+        // Force effects have changed
+        return true;
+      }
+    }
+
+    // Compare shapes
+    if ( tmp_shapes.size() == current_shapes.size() ) {
+      for ( size_t i= 0; i < tmp_shapes.size(); ++i ) {
+        if ( tmp_shapes[i].size() == current_shapes[i].size() ) {
+          for ( size_t j= 0; j < tmp_shapes[i].size(); ++j ) {
+            if ( tmp_shapes[i][j] != current_shapes[i][j] ) {
+              // Shapes have changed
+              return true;
+            }
+          }
+        }
+      }
+    }
+
+    // Every thing is the same
+    return false;
+  } else {
+    // Force effects have changed
+    return true;
+  }
+}
\ No newline at end of file
