@@ -1,5 +1,5 @@
 //////////////////////////////////////////////////////////////////////////////
-//    Copyright 2004-2014, SenseGraphics AB
+//    Copyright 2004-2019, SenseGraphics AB
 //
 //    This file is part of HAPI.
 //
@@ -109,7 +109,7 @@ void RuspiniRenderer::onOnePlaneContact( const Vec3& proxy_pos,
   c.haptic_shape->getSurface()->getForces( contact );
   
   // add a contact
-  tmp_contacts.push_back( make_pair( c.haptic_shape, contact ) );
+  tmp_contacts.push_back( std::make_pair( c.haptic_shape, contact ) );
 }
 
 void RuspiniRenderer::onTwoPlaneContact( const Vec3& proxy_pos,
@@ -222,21 +222,21 @@ void RuspiniRenderer::onTwoPlaneContact( const Vec3& proxy_pos,
     contact.force_global = p0_force * weight + p1_force * ( 1 - weight );
 
     // add contacts
-    tmp_contacts.push_back( make_pair( p0.haptic_shape, contact ) );
+    tmp_contacts.push_back( std::make_pair( p0.haptic_shape, contact ) );
     if( p0.haptic_shape.get() != p1.haptic_shape.get() )
-      tmp_contacts.push_back( make_pair( p1.haptic_shape, contact ) );
+      tmp_contacts.push_back( std::make_pair( p1.haptic_shape, contact ) );
   }
 }
 
 
 void RuspiniRenderer::onThreeOrMorePlaneContact(  
           const Vec3& proxy_pos,
-          Constraints &constraints,
+          Constraints &_constraints,
           HAPISurfaceObject::ContactInfo &contact ) {
-  assert( constraints.size() >= 3 );
+  assert(_constraints.size() >= 3 );
 
   // find the first three planes that all constrain the proxy
-  Constraints::iterator i = constraints.begin();
+  Constraints::iterator i = _constraints.begin();
   PlaneConstraint &p0 = (*i++);
   PlaneConstraint &p1 = (*i++);
   PlaneConstraint &p2 = (*i++);
@@ -244,7 +244,7 @@ void RuspiniRenderer::onThreeOrMorePlaneContact(
   Vec3 contact_global = contact.globalContactPoint();
   Vec3 probe_local_pos( 0, 0, 0 );
 
-  if( i == constraints.end() ) {
+  if( i == _constraints.end() ) {
     // transformation matrix from local coordinate system with the normals
     // of the constraint planes as axis to global space
     Matrix4 m( p0.normal.x, p1.normal.x, p2.normal.x, contact_global.x, 
@@ -261,12 +261,9 @@ void RuspiniRenderer::onThreeOrMorePlaneContact(
       // to trigger the "three-plane-constraint" code further down. I guess theoretically
       // this case should result in a movement along the normal perpendicular to
       // the plane spanned by the three normals, but this is a much faster fix.
-      /// \todo Implement correct case for when three normals are in the same plane?
-      /// Not very likely to happen unless FakeHapticsDevice is used with surface
-      /// that changes. probe_local_pos is already 0,0,0 so do nothing here.
     }
   } else {
-    while( i != constraints.end() ) {
+    while( i != _constraints.end() ) {
       // transformation matrix from local coordinate system with the normals
       // of the constraint planes as axis to global space
       Matrix4 m( p0.normal.x, p1.normal.x, p2.normal.x, contact_global.x, 
@@ -387,14 +384,14 @@ void RuspiniRenderer::onThreeOrMorePlaneContact(
 
 
     // add contacts
-    tmp_contacts.push_back( make_pair( p0.haptic_shape, contact ) );
+    tmp_contacts.push_back( std::make_pair( p0.haptic_shape, contact ) );
     if( p0.haptic_shape.get() != p1.haptic_shape.get() ) {
-      tmp_contacts.push_back( make_pair( p1.haptic_shape, contact ) );
+      tmp_contacts.push_back( std::make_pair( p1.haptic_shape, contact ) );
     }
     
     if( p0.haptic_shape.get() != p2.haptic_shape.get() &&
         p1.haptic_shape.get() != p2.haptic_shape.get() ) {
-      tmp_contacts.push_back( make_pair( p2.haptic_shape, contact ) );
+      tmp_contacts.push_back( std::make_pair( p2.haptic_shape, contact ) );
     }
   }
 }
@@ -433,7 +430,6 @@ RuspiniRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
         //
         // Pontus added: "|| alwaysFollowSurface" to override this check. If not set, the proxy will not move along
         // with a moving rigid body when "dragging" it using lateral friction force.
-        // Todo: Investigate if problems occur, do this check in a cleaner way.
         if( (proxy_pos - moved_proxy_pos).dotProduct( proxy_pos - input.position ) <= 0 || alwaysFollowSurface ) {
           proxy_pos = moved_proxy_pos;
           done = true;
@@ -564,8 +560,7 @@ RuspiniRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
 
   unsigned int nr_constraints = closest_constraints.size();
 
-  // TODO: Could this check be included in the first loop somehow
-  // or will that just slow it down more. Without this part the
+  // Without this part the
   // proxy will be stuck between two planes in the part where the angle
   // is below pi/4
   if( nr_constraints > 0 ) {
@@ -605,7 +600,7 @@ RuspiniRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
     }
   }
 
-  Vec3 new_proxy_pos, new_force;
+  Vec3 new_proxy_pos, new_force, new_torque;
 
   HAPISurfaceObject::ContactInfo contact;
   contact.hd = hd;
@@ -634,6 +629,7 @@ RuspiniRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
     new_proxy_pos = contact.globalOrigin();
 
     new_force = contact.force_global;
+    new_torque = contact.torque_global;
   }
     
   // In order to prevent fallthrough of moving shapes when holding the proxy
@@ -661,8 +657,8 @@ RuspiniRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
         // the shape will probably come into contact with the object between now
         // and next time step so we move the proxy with the shape to not miss it.
         Vec3 new_target = (*i)->moveTimestep( new_proxy_pos, -dt );
-        Collision::IntersectionInfo intersection;
-        if( (*i)->lineIntersect( new_proxy_pos, new_target, intersection,
+        Collision::IntersectionInfo intersection_tmp;
+        if( (*i)->lineIntersect( new_proxy_pos, new_target, intersection_tmp,
                                  (*i)->getTouchableFace()) ){
           new_proxy_pos = (*i)->moveTimestep( new_proxy_pos, dt );
         }
@@ -675,12 +671,13 @@ RuspiniRenderer::renderHapticsOneStep( HAPIHapticsDevice *hd,
   last_contact_transforms.clear();
   for( unsigned int i = 0; i < tmp_contacts.size(); ++i ) {
     if( tmp_contacts[i].first->isDynamic() ) {
-      last_contact_transforms.push_back( make_pair( tmp_contacts[i].first->getShapeId(),
+      last_contact_transforms.push_back( std::make_pair( tmp_contacts[i].first->getShapeId(),
                                                     tmp_contacts[i].first->getInverse() ) );
     }
   }
 
   output.force = new_force;
+  output.torque = new_torque;
   proxy_info_lock.lock();
   proxy_position = new_proxy_pos;
   proxy_info_lock.unlock();
